require "easy/configuration/railtie"

module Easy
  module Configuration
    extend ActiveSupport::Autoload

    autoload :Structure
    autoload :Auth
    autoload :Autocomplete
    autoload :Custom
    autoload :Entity
    autoload :Menu
    autoload :MenuItem
    autoload :MenuTitle
    autoload :MenuEntityLink
    autoload :MenuCustomLink

    module Patches
      extend ActiveSupport::Autoload

      autoload :ActiveRecord
    end

    cattr_accessor :application_name
    self.application_name = 'Easy'

    cattr_accessor :application_icon
    self.application_icon = 'application_icon'

    cattr_accessor :email_default_from
    self.email_default_from = 'info@easysoftware.com'

    cattr_accessor :protocol
    self.protocol = 'https'

    cattr_accessor :host_name
    self.host_name = 'app.easysoftware.com'

    cattr_accessor :_entities
    self._entities = {}

    cattr_accessor :_menu
    self._menu = {}

    class << self

      def host
        "#{protocol}://#{host_name}"
      end

      def entity(klass_or_name, &block)
        k = klass_or_name.is_a?(String) ? klass_or_name : klass_or_name.name.underscore

        self._entities[k] ||= Entity.new(k)

        if block_given?
          yield self._entities[k]
        else
          self._entities[k]
        end
      end

      def menu(section, &block)
        self._menu[section.to_s] ||= Menu.new

        if block_given?
          yield self._menu[section.to_s]
        else
          self._menu[section.to_s]
        end
      end

      def auth(&block)
        if block_given?
          yield Auth
        else
          Auth
        end
      end

      def sites
        @sites ||= begin
          h = ::ActiveSupport::OrderedOptions.new
          h.merge!(secrets.sites || {})
          h
        end
      end

      def custom
        @custom ||= Custom.new
      end

      def secrets
        Rails.application.secrets
      end

    end

  end

  class << self

    def config
      ::Easy::Configuration
    end

    def configure
      yield config
    end

  end
end
