module Easy
  module Configuration
    class Menu

      attr_accessor :visibility
      attr_writer :caption

      def initialize
        @items   = {}
        @caption = nil
      end

      def visible?(current_user)
        case @visibility
        when Proc
          @visibility.call(current_user)
        when NilClass
          true
        else
          !!@visibility
        end
      end

      def caption
        case @caption
        when String
          @caption
        when NilClass
          ''
        else
          I18n.t(@caption)
        end
      end

      def custom_link(key, options = {}, &block)
        parent = options.delete(:parent)

        add(key, MenuCustomLink.new(options, &block), parent)
      end

      def entity_link(class_name, options = {}, &block)
        parent = options.delete(:parent)

        add(class_name, MenuEntityLink.new(class_name, options, &block), parent)
      end

      def title(key, options = {}, &block)
        parent = options.delete(:parent)

        add(key, MenuTitle.new(options, &block), parent)
      end

      def items
        @items.values
      end

      private

      def add(key, menu_item, parent = nil)
        if parent && @items[parent.to_s]
          @items[parent.to_s].children[key.to_s] = menu_item
        else
          @items[key.to_s] = menu_item
        end
      end

    end
  end
end
