module Easy
  module Configuration
    class Structure

      def initialize(default_values = {})
        @values  = default_values.dup
        @changed = Hash.new { |hash, key| hash[key] = false }

        define_instance_methods
      end

      def merge!(structure, ignore_defaults = true)
        if ignore_defaults
          @values.merge!(structure.values) { |key, v1, v2| structure.changed?(key) ? v2 : v1 }
        else
          @values.merge!(structure.values)
        end
      end

      protected

      def values
        @values
      end

      def changed?(attribute)
        @changed[attribute]
      end

      private

      def define_instance_methods
        @values.keys.each do |key|
          self.class.send :define_method, key do
            @values[key]
          end

          self.class.send :define_method, "#{key}_changed?" do
            @changed[key]
          end

          self.class.send :define_method, "#{key}=" do |v|
            @changed[key] = true
            @values[key]  = v
          end
        end
      end

    end
  end
end
